function [R,tix]=nan_histo4(Y)
//  calculates histogram for rows and supports data compression
// Calling Sequence
// R = nan_histo4(Y)
// [R,tix] = nan_histo4(Y) 
// Parameters
// 	R : is a struct with th fields 
//       R.X : are the bin-values 
//       R.H : is the frequency of occurence of value X 
//  	R.N : are the number of samples 
//  	tix : enables compression
//  	 R.X(tix,:) : is the decompression step
// Description
// histo4 might be useful for data compression, because
//
// The effort (in memory and speed) for compression is O(n*log(n))
// The effort (in memory and speed) for decompression is only O(n)
// Examples
// x= [ 9 9 9 9 2 2 3 3 4 5 9 ]';
// [R,tix]=nan_histo4(x)
// disp(R.X(tix,:))
// Bibliography
//  C.E. Shannon and W. Weaver "The mathematical theory of communication" University of Illinois Press, Urbana 1949 (reprint 1963).
// See also
//    nan_histo
//    nan_histo2
//    nan_histo3
// Authors
//	Copyright (C) 1996-2005,2008 by Alois Schloegl a.schloegl@ieee.org
//	H. Nahrstaedt - 2010


//	$Id: histo4.m 5090 2008-06-05 08:12:04Z schloegl $
//	Copyright (C) 1996-2005,2008 by Alois Schloegl <a.schloegl@ieee.org>	
//    	This is part of the TSA-toolbox 
//	http://hci.tugraz.at/~schloegl/matlab/tsa/
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 [nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least 1 parameter required');
end;

[yr, yc] = size(Y);
if yr==1,
        // Makes sure there is a second row
        // Sort does not support the DIM-argument, therefore,
        // this function would not work correctly with this software
        // Once this is fixed, this part can be removed. 
        Y = [Y; %nan+ones(Y)];  
end;


// identify all possible X's and overall Histogram
[Y,   idx] = gsort(Y,'r','i');
[tmp, idx] = mtlb_sort(idx);        // inverse index

//[ix, iy] = (diff(Y,1)>0);
ix = (zeros(yr-1,1)~=0);
for k = 1:yr-1,
        //ix(k) = or(Y(k,:)~=Y(k+1,:));
     d = Y(k,:)-Y(k+1,:);
     d = (~isnan(d) & d~=0) | (isnan(Y(k,:)) ~= isnan(Y(k+1,:)));
     ix(k) = or(d);
end;

tmp = [find(ix), yr]';

R.datatype = 'HISTOGRAM';
R.H = diff([0; tmp]);
R.X = Y(tmp,:);
R.N = yr;


// generate inverse index
if nargout>1,
        tix = cumsum([1;ix]);	// rank 
        cc  = 1;
        tmp = sum(ix)+1;
        if tmp <= 2^8;
                tix = uint8(tix);
                cc = 8/1;
        elseif tmp <= 2^16;
                tix = uint16(tix);
                cc = 8/2;
        elseif tmp <= 2^32;
                tix = uint32(tix);
                cc = 8/4;
        end;
        tix = tix(idx);		// inverse sort rank
        
        R.compressionratio = (prod(size(R.X)) + yr/cc) / (yr*yc);
        R.tix = tix;
end;
endfunction
